#ifndef FAST_MATRIX_INC
#define FAST_MATRIX_INC

#include "Object.h"

namespace Torch {

/** This class is designed to enable fast vector-matrix operations for small sizes (from 2 to 5)

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class fastMatrix : public Object
{
	void (*initVector_)(int, real *);
	void (*initZeroMatrix_)(int, real **);
	void (*initMatrix_)(int, real **, real);
	void (*transposeMatrix_)(int, real **);
	void (*copyVector_)(int, real *, real *);
	void (*copyMatrix_)(int, real **, real **);
	void (*vectorialProduct_)(int, real **, real *, real *);
	void (*product_)(int, real *, real *, real **);
	void (*addMatrix_)(int, real **, real **);
	void (*prodMatrix_)(int, real **, real);
	void (*divMatrix_)(int, real **, real);
	void (*divVector_)(int, real *, real);
	real (*normVector_)(int, real *);
	real (*normMatrix_)(int, real **);
	real (*inverse_)(int, real **Minv, real **M1);

public:
   	/**@name fields */
        //@{
   	/// matrix size
   	int n;
	//@}
	
	//-----
	
   	/**@name constructor and destructor */
        //@{
	/** Creates a set of fast vector-matrix operations of a given size
	
	    @param n_ is the size of the matrix
	*/
    	fastMatrix(int n_);
	
	/// destructor
	~fastMatrix();
	//@}
	
   	/**@name vector and matrix initialization */
        //@{
	/// Zero-ing the vector #V_#
	void initVector(real *V_);

	/// Zero-ing the matrix #M_#
	void initMatrix(real **M_);
	
	/// Filling all the matrix #M_# with #d_#
	void initMatrix(real **M_, real d_);
	//@}
	
   	/**@name vector and matrix operations */
        //@{
	/// Transpose the matrix #M_#
	void transposeMatrix(real **M_);

	/// Copy vector #V2_# to #V1_#
	void copyVector(real *V1_, real *V2_);

	/// Copy matrix #M2_# to #M1_#
	void copyMatrix(real **M1_, real **M2_);

	/** Computes the vectorial product
	
	    @param M_ is the matrix that contains $X \cdot X^T$
	    @param X_ is a vector
	    @param T_X_ is the transpose of vector $X$
	*/
	void vectorialProduct(real **M_, real *X_, real *T_X_);

	/** Computes the vector-matrix product
	
	    @param VM_ is the vector that contains $V \cdot M$
	    @param V_ is a vector
	    @param M_ is a matrix
	*/
	void product(real *VM_, real *V_, real **M_);

	/** Computes the addition of 2 matrices
	
	    @param M_ is the matrix that contains $M_{ij} + Madd_{ij} \, \forall i,j = 1 \dots n$
	    @param Madd_ is a matrix
	*/
	void addMatrix(real **M_, real **Madd_);
	
	/** Computes the product of a matrix and a scalar
	
	    @param M_ is the matrix that contains $M_{ij} \times d \, \forall i,j = 1 \dots n$
	    @param d_ is a scalar
	*/
	void prodMatrix(real **M_, real d_);
	
	/** Computes the division of a matrix and a scalar
	
	    @param M_ is the matrix that contains $\frac{M_{ij}}{d} \, \forall i,j = 1 \dots n$
	    @param d_ is a scalar
	*/
	void divMatrix(real **M_, real d_);
	
	/** Computes the division of a vector and a scalar
	
	    @param V_ is the vector that contains $\frac{V_i}{d} \, \forall i = 1 \dots n$
	    @param d_ is a scalar
	*/
	void divVector(real *V_, real d_);

	/** Computes the inverse of a matrix
	
	    @param Minv_ contains $M^{-1}$
	    @param M_ is a matrix
	    @return the determinant
	*/
	real inverse(real **Minv_, real **M_);

	/** Computes the norm of a vector
	
	    @param V_ is a vector
	    @return the norm of the vector $\sqrt{\sum_{i=1}^n V_i^2}$

	*/
	real normVector(real *V_);

	/** Computes the norm of a matrix
	
	    @param M_ is a matrix
	    @return the norm of the matrix $\frac{1}{n} \sqrt{\sum_{i=1}^n \sum_{j=1}^n M_{i,j}^2} \, \forall i,j 1 \dots n$

	*/
	real normMatrix(real **M_);
	//@}

   	/**@name other methods */
        //@{
	/// print the vector #V_#
	void printVector(real *V_);

	/// print the matrix #M_#
	void printMatrix(real **M_);
	//@}
};

void initVector2x2_(int n, real *V);
void initZeroMatrix2x2_(int n, real **M);
void initMatrix2x2_(int n, real **M, real d);
void copyVector2x2_(int n, real *V1, real *V2);
void copyMatrix2x2_(int n, real **M1, real **M2);
void vectorialProduct2x2_(int n, real **M, real *X, real *T_X);
void product2x2_(int n, real *VM, real *V, real **M);
void addMatrix2x2_(int n, real **M, real **Madd);
void prodMatrix2x2_(int n, real **M, real d);
void divMatrix2x2_(int n, real **M, real d);
void divVector2x2_(int n, real *V, real d);
real normVector2x2_(int n, real *V);
real normMatrix2x2_(int n, real **M);
real inverse2x2_(int n, real **Minv, real **M1);

void initVector3x3_(int n, real *V);
void initZeroMatrix3x3_(int n, real **M);
void initMatrix3x3_(int n, real **M, real d);
void copyVector3x3_(int n, real *V1, real *V2);
void copyMatrix3x3_(int n, real **M1, real **M2);
void vectorialProduct3x3_(int n, real **M, real *X, real *T_X);
void product3x3_(int n, real *VM, real *V, real **M);
void addMatrix3x3_(int n, real **M, real **Madd);
void prodMatrix3x3_(int n, real **M, real d);
void divMatrix3x3_(int n, real **M, real d);
void divVector3x3_(int n, real *V, real d);
real normVector3x3_(int n, real *V);
real normMatrix3x3_(int n, real **M);
real inverse3x3_(int n, real **Minv, real **M1);

void initVector4x4_(int n, real *V);
void initZeroMatrix4x4_(int n, real **M);
void initMatrix4x4_(int n, real **M, real d);
void copyVector4x4_(int n, real *V1, real *V2);
void copyMatrix4x4_(int n, real **M1, real **M2);
void addMatrix4x4_(int n, real **M, real **Madd);
void prodMatrix4x4_(int n, real **M, real d);
void divMatrix4x4_(int n, real **M, real d);
void divVector4x4_(int n, real *V, real d);
real normVector4x4_(int n, real *V);
real normMatrix4x4_(int n, real **M);

void initVectornxn_(int n, real *V);
void initZeroMatrixnxn_(int n, real **M);
void initMatrixnxn_(int n, real **M, real d);
void transposeMatrixnxn_(int n, real **M);
void copyVectornxn_(int n, real *V1, real *V2);
void copyMatrixnxn_(int n, real **M1, real **M2);
void vectorialProductnxn_(int n, real **M, real *X, real *T_X);
void productnxn_(int n, real *VM, real *V, real **M);
void addMatrixnxn_(int n, real **M, real **Madd);
void prodMatrixnxn_(int n, real **M, real d);
void divMatrixnxn_(int n, real **M, real d);
void divVectornxn_(int n, real *V, real d);
real normVectornxn_(int n, real *V);
real normMatrixnxn_(int n, real **M);
real inversenxn_(int n, real **Minv, real **M1);

}

#endif
