#include "ipVcycle.h"
#include "ImageGray.h"
#include "multigrid.h"

//*******************************************************************
//
// This file implements the multigrid V-cycle algorithm from 
// "a multigrid tutorial"[Briggs] 
//
// ---> author: Guillaume Heusch (heusch@idiap.ch) <---
//
//*******************************************************************

namespace Torch {

  ipVcycle::ipVcycle(real lambda_, int n_grids_, int type_, int width_, int height_, const char *coding_) : ipCore(coding_)
  {
    width = width_;
    height = height_;
    
    // relative importance of the smoothness constraint
    lambda = lambda_;

    // number of grids 
    n_grids = n_grids_;

    // type of diffusion (coefficients)
    type = type_;
    
    // small array to store the diffusion coefficients for each processed pixel
    rho = (real*) allocator->alloc(sizeof(real)*5);
    
    // output
    seq_out = new(allocator) Sequence(2, width*height);
    
    for(int i=0; i<seq_out->n_frames; i++)
      for(int j=0; j<seq_out->frame_size; j++)
	seq_out->frames[i][j] = 0.0; 
    
    // result of the MGV
    light = new(allocator) Vec(width*height);

    // the original image
     image_grid = new(allocator) Vec(width*height);
  }


  void ipVcycle::process(Sequence *seq_in)
  {
    switch(coding_id)
      {
      case 0:
	{

	  //copy the data into the finest image grid
 	  for (int i=0; i<width*height; i++) 
 	    image_grid->ptr[i] = seq_in->frames[0][i];

	  // take zero as an initial guess
	  Vec *guess = new(allocator) Vec(width*height);
	  guess->zero();
	
	  // call to Multigrid V-cycle
	  light = mgv(guess, image_grid, lambda, 0, type);

	  // build final result (R = I/L)
	  for (int i=0; i<width*height; i++) {
	    
	    if ((i < width) || (i >= ((height - 1)*width)) ||  ((i % width) < 1) || ((i % width) >= (width-1))) {
	      seq_out->frames[0][i] = 1;
	    }
	    
	    else {
	      
	      if (IS_NEAR(light->ptr[i], 0.0, 0.01)) {
		seq_out->frames[0][i] = 1;
	      }
	      else{
		seq_out->frames[0][i] = ((image_grid->ptr[i])/(light->ptr[i]));
	      }
	    }
	    seq_out->frames[1][i] = light->ptr[i];
	  }
	  
	  // display purpose
	  cutExtremum(seq_out->frames[0], width, height, 4);
	  
	  // rescale between 0-255
	  scaleGray(width, height, seq_out->frames[0]);
	  
	  break;
	}

      case 1:
	message("ipVcycle::process() not implemented for RGB format");
	break;
      case 2:
	message("ipVcycle::process() not implemented for YUV format");
	break;
      case 3:
	message("ipVcycle::process() not implemented for float format");
	break;
      }
  }


  Vec* ipVcycle::mgv(Vec *x, Vec *b, real lambda, int level, int type) {

    Vec *result = new(allocator) Vec(width*height);

    // if we are on the coarsest level -> solve
    if (level == n_grids-1) {
   
      diffOperator = new(allocator) Mat(width*height, width*height);
      buildOperator(diffOperator, rho, lambda, type, b, width, height);
     
      mxSolve(diffOperator, b, result); 
      
      // set boundary pixels to zero
      for (int i=0; i<width*height; i++) {

	bool is_on_boundary = (i < width) || (i >= (height-1)*width) || (i % width == 0) || (i % width == (width-1));
	
	if (is_on_boundary)
	    result->ptr[i] = 0.0;
      }
    }
    
    else {
      
      // relax (pre-smooth) 
      gaussSeidel(x, b, rho, lambda, type, width, height);
      
      // compute residual
      Vec *temp = new(allocator) Vec(width*height);	
      myMultiply(x, temp, rho, lambda, type, width, height);
      for (int i=0; i<width*height; i++) 
	temp->ptr[i] = b->ptr[i] - temp->ptr[i];
      
      // restrict residual
      Vec *rhat = new(allocator) Vec((width/2)*(height/2)); 
      restrict(temp, rhat, width, height);
      width /= 2;
      height /= 2;

      // recursive call to mgv
      Vec *xhat = new(allocator) Vec(width*height);
      Vec *zero = new(allocator) Vec(width*height);
      zero->zero();
      xhat = mgv(zero, rhat, lambda, level+1, type);
      
      // interpolate correction
      Vec *xcorr = new(allocator) Vec((width*2)*(height*2)); 
      project(xhat, xcorr, width ,height);
      width *= 2;
      height *= 2;

      // update solution
      for (int i=0; i<width*height; i++) 
	result->ptr[i] = x->ptr[i] + xcorr->ptr[i]; 

      // relax (post-smooth)
      gaussSeidel(result, b, rho, lambda, type, width, height);
    } 
 
    return result;
  }



  void ipVcycle::cutExtremum(real *data, int width, int height, int distribution_width) {
    
    // used to 'cut' the extreme of the pixels distribution in the result
    real mean_out = 0.0;
    real var_out = 0.0;
    real std_dev = 0.0; 
    
    // compute the mean
    for (int i=0; i<width*height; i++)
      mean_out += data[i];   
    mean_out /= width*height;
    
  
    // compute variance and standard deviation
    for (int i=0; i<width*height; i++)
      var_out += (data[i]- mean_out)*(data[i]- mean_out);    
    var_out /= (width*height - 1);
    std_dev = sqrt(var_out);

    
    int count = 0;
    
    for (int i=0; i<width*height; i++) {
      
      if (data[i] > mean_out + distribution_width*std_dev) {
	data[i] = mean_out + distribution_width*std_dev;
	count++;	
      }
      
      if (data[i] < mean_out - distribution_width*std_dev){
	data[i] = mean_out - distribution_width*std_dev;
	count++;
      }
    }
  }
  

  ipVcycle::~ipVcycle()
  {
  }

}
