#include "MyMeanVarNorm.h"
#include "XFile.h"

namespace Torch {

MyMeanVarNorm::MyMeanVarNorm(int n_inputs_, int n_targets_, bool norm_inputs, bool norm_targets)
{
  inputs_mean = NULL;
  inputs_stdv = NULL;
  targets_mean = NULL;
  targets_stdv = NULL;
  n_inputs = n_inputs_;
  n_targets = n_targets_;

  if(norm_inputs)
  {
    inputs_mean = (real *)allocator->alloc(sizeof(real)*n_inputs);
    inputs_stdv = (real *)allocator->alloc(sizeof(real)*n_inputs);
    for(int i = 0; i < n_inputs; i++)
    {
      inputs_mean[i] = 0;
      inputs_stdv[i] = 0;
    }
  }

  if(norm_targets)
  {
    targets_mean = (real *)allocator->alloc(sizeof(real)*n_targets);
    targets_stdv = (real *)allocator->alloc(sizeof(real)*n_targets);
    for(int i = 0; i < n_targets; i++)
    {
      targets_mean[i] = 0;
      targets_stdv[i] = 0;
    }
  }

}

void MyMeanVarNorm::normalizeSequence(Sequence *sequence, real *mean, real *stdv)
{
  for(int i = 0; i < sequence->n_frames; i++)
  {
    real *ptr_ = sequence->frames[i];
    for(int k = 0; k < sequence->frame_size; k++)
      ptr_[k] = (ptr_[k] - mean[k])/stdv[k];
  }
}

void MyMeanVarNorm::preProcessInputs(Sequence *inputs)
{
  if(!inputs_mean)
    return;

  normalizeSequence(inputs, inputs_mean, inputs_stdv);
}

void MyMeanVarNorm::preProcessTargets(Sequence *targets)
{
  if(!targets_mean)
    return;

  normalizeSequence(targets, targets_mean, targets_stdv);
}

void MyMeanVarNorm::loadXFile(XFile *file)
{
  if(inputs_mean)
  {
    file->taggedRead(inputs_mean, sizeof(real), n_inputs, "IMEANS");
    file->taggedRead(inputs_stdv, sizeof(real), n_inputs, "ISTDVS");
  }
  if(targets_mean)
  {
    file->taggedRead(targets_mean, sizeof(real), n_targets, "TMEANS");
    file->taggedRead(targets_stdv, sizeof(real), n_targets, "TSTDVS");
  }
}

void MyMeanVarNorm::saveXFile(XFile *file)
{
  if(inputs_mean)
  {
    file->taggedWrite(inputs_mean, sizeof(real), n_inputs, "IMEANS");
    file->taggedWrite(inputs_stdv, sizeof(real), n_inputs, "ISTDVS");
  }
  if(targets_mean)
  {
    file->taggedWrite(targets_mean, sizeof(real), n_targets, "TMEANS");
    file->taggedWrite(targets_stdv, sizeof(real), n_targets, "TSTDVS");
  }
}

void MyMeanVarNorm::normalize(int n_total_frames)
{
  if(inputs_mean)
  {
    for(int i = 0; i < n_inputs; i++)
    {
      inputs_mean[i] /= (real)n_total_frames;
      inputs_stdv[i] /= (real)n_total_frames;
      inputs_stdv[i] -= inputs_mean[i]*inputs_mean[i];
      if(inputs_stdv[i] <= 0)
      {
        warning("CustomMeanVarNorm: input column %d has a null stdv. Replaced by 1.", i);
        inputs_stdv[i] = 1.;
      }
      else
        inputs_stdv[i] = sqrt(inputs_stdv[i]);
    }
  }

  if(targets_mean)
  {
    for(int i = 0; i < n_targets; i++)
    {
      targets_mean[i] /= (real)n_total_frames;
      targets_stdv[i] /= (real)n_total_frames;
      targets_stdv[i] -= targets_mean[i]*targets_mean[i];
      if(targets_stdv[i] <= 0)
      {
        warning("CustomMeanVarNorm: target column %d has a null stdv. Replaced by 1.", i);
        targets_stdv[i] = 1.;
      }
      else
        targets_stdv[i] = sqrt(targets_stdv[i]);
    }
  }
}

MyMeanVarNorm::~MyMeanVarNorm()
{
}

}
