#ifndef SERIAL_INC
#define SERIAL_INC

#include "general.h"

#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/termios.h>
#include <unistd.h>
#include <errno.h>

namespace Torch {

#define MAXPACKETLENGTH 15
#define RETRIES 3

/** This class is designed to manage a serial port

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class Serial 
{
public:
   	/// verbose serial communication if true
	bool bVerbose;
   
   	// Packet structure
	typedef struct
	{
		unsigned char packet[MAXPACKETLENGTH];
		int length;
	} sCommandPacket;
	
   	/// the device (/dev/ttyS0 or /dev/ttya for instance)
	char *device;

	/// serial port handle
	int handle;
	
	//-----
	
	///
   	Serial(char *device_, char port_);

	///
	virtual ~Serial();

	/// connect to the serial port
	bool connect();

	/// specific device initialisation
	virtual bool init() = 0;

	/// print the command in hexadecimal
	void printCmd(sCommandPacket *command);
	   
	/** send the #command# on the serial port. 
		If #ack# is true then wait for acknowledgement.
		If #cpl# is true then wait for completion.
	*/
	bool sendCmd(sCommandPacket *command, bool ack = true, bool cpl = true);

	/** send the #command# without acknowledgement and completion.
		Returns the number of bytes written.
	*/
	int send(sCommandPacket *command);
	
	/** read command from the serial port.
		Returns the number of bytes which were read.
	*/
	int readCmd(sCommandPacket *command);

	/** wait max timeInSec seconds for #command#.
		Returns the number of written bytes if success, -1 otherwise.
	*/
	int waitingForReply(int timeInSec, sCommandPacket *command);

	/// is #command1# equal #command2# 
	int isEqual(sCommandPacket *command1, sCommandPacket *command2);
	   
	/// is #command# an acknowledgement packet
	virtual bool IsAck(sCommandPacket *command) = 0;
	
	/// is #command# an completion packet
	virtual bool IsCpl(sCommandPacket *command) = 0;

	//-----
	
	/// terminator at the end of packets
	virtual unsigned char byteTerminator() = 0;
};

}

#endif
