#include "DiskBinDataSet.h"
#include "IOMulti.h"
#include "IOBinDisk.h"
#include "IOVirtual.h"

namespace Torch {

DiskBinDataSet::DiskBinDataSet(const char *filename, int n_inputs_, real target_, int max_load)
{
        if(n_inputs_ <= 0)
		error("DiskBinDataSet: n_inputs must be positive!");

	//	
	// Firstly the inputs
	IOBinDisk *io_file = new(allocator) IOBinDisk(filename, max_load);
	io_inputs = io_file; 
	
	if(io_inputs->frame_size != n_inputs_)
		error("DiskBinDataSet: the n_inputs_ provided doesn't correspond to the DimIn of the bindata file");
	
	//
	// then the targets (same real target for all patterns)
	IOVirtual *io_virtual = new(allocator) IOVirtual(target_, io_file->n_sequences);
	io_targets = io_virtual;
	
	//
	DiskDataSet::init(io_inputs, io_targets);  
	message("DiskBinDataSet: %d examples scanned assigned with target %g", n_examples, target_);
}
	
DiskBinDataSet::DiskBinDataSet(const char *filename,  int n_inputs_, int max_load)
{
        if(n_inputs_ <= 0)
		error("DiskBinDataSet: n_inputs must be positive!");

	//	
	// Here we have only the inputs
	IOBinDisk *io_file = new(allocator) IOBinDisk(filename, max_load);
	io_inputs = io_file; 
	
	if(io_inputs->frame_size != n_inputs_)
		error("DiskBinDataSet: the n_inputs_ provided doesn't correspond to the DimIn of the bindata file");
	
	//
	DiskDataSet::init(io_inputs, NULL);   
	message("DiskBinDataSet: %d examples scanned with no targets assigned", n_examples);
}

DiskBinDataSet::DiskBinDataSet(char **filenames, int n_files, int n_inputs_, real *targets_, int max_load)
{
	if(n_files <= 0)
		error("DiskBinDataSet: check the number of files!");

	if(n_inputs_ <= 0)
		error("DiskBinDataSet: n_inputs must be positive!");

	//
	// Firstly the inputs
	IOBin **io_files = (IOBin **)allocator->alloc(sizeof(IOBin *)*n_files);
	for(int i = 0; i < n_files; i++)
		//io_files[i] = new(allocator) IOBin(filenames[i], false, max_load, false);
		io_files[i] = new(allocator) IOBinDisk(filenames[i], max_load);

	IOMulti *io_fileI = new(allocator) IOMulti((IOSequence **)io_files, n_files);
	io_inputs = io_fileI;  
	
	if(io_inputs->frame_size != n_inputs_)
		error("DiskBinDataSet: the n_inputs_ provided doesn't correspond to the DimIn of the bindata file");
	
	//
	// then the targets (same real target for all patterns of the same file[i])
	IOVirtual **io_virtuals = (IOVirtual **)allocator->alloc(sizeof(IOVirtual *)*n_files);
	for(int i = 0; i < n_files; i++)
		io_virtuals[i] = new(allocator) IOVirtual(targets_[i], io_files[i]->n_sequences);

	IOMulti *io_fileT = new(allocator) IOMulti((IOSequence **)io_virtuals, n_files);
	io_targets = io_fileT;  


	//
	DiskDataSet::init(io_inputs, io_targets);
	message("DiskBinDataSet: %d examples scanned from %d files with targets assigned from an array", n_examples, n_files);
}

DiskBinDataSet::DiskBinDataSet(char **filenames, int n_files, int n_inputs_, int max_load)
{
	if(n_files <= 0)
		error("DiskBinDataSet: check the number of files!");

	if(n_inputs_ <= 0)
		error("DiskBinDataSet: n_inputs must be positive!");

	//
	// Firstly the inputs
	IOBin **io_files = (IOBin **)allocator->alloc(sizeof(IOBin *)*n_files);
	for(int i = 0; i < n_files; i++)
		//io_files[i] = new(allocator) IOBin(filenames[i], false, max_load, false);
		io_files[i] = new(allocator) IOBinDisk(filenames[i], max_load);

	IOMulti *io_fileI = new(allocator) IOMulti((IOSequence **)io_files, n_files);
	io_inputs = io_fileI;  
	
	if(io_inputs->frame_size != n_inputs_)
		error("DiskBinDataSet: the n_inputs_ provided doesn't correspond to the DimIn of the bindata file");
	
	//
	DiskDataSet::init(io_inputs, NULL);
	message("DiskBinDataSet: %d examples scanned from %d files with no targets assigned", n_examples, n_files);
}

DiskBinDataSet::DiskBinDataSet(char **filenames, int n_files, int n_inputs_, real target_, int max_load)
{
	if(n_files <= 0)
		error("DiskBinDataSet: check the number of files!");

	if(n_inputs_ <= 0)
		error("DiskBinDataSet: n_inputs must be positive!");

	//
	// Firstly the inputs
	IOBin **io_files = (IOBin **)allocator->alloc(sizeof(IOBin *)*n_files);
	for(int i = 0; i < n_files; i++)
		//io_files[i] = new(allocator) IOBin(filenames[i], false, max_load, false);
		io_files[i] = new(allocator) IOBinDisk(filenames[i], max_load);

	IOMulti *io_fileI = new(allocator) IOMulti((IOSequence **)io_files, n_files);
	io_inputs = io_fileI;  
	
	if(io_inputs->frame_size != n_inputs_)
		error("DiskBinDataSet: the n_inputs_ provided doesn't correspond to the DimIn of the bindata file");
	
	//
	// then the targets (same real target for all patterns)
	IOVirtual *io_virtual = new(allocator) IOVirtual(target_, io_fileI->n_sequences);
	io_targets = io_virtual;
	
	//
	DiskDataSet::init(io_inputs, io_targets);  
	message("DiskBinDataSet: %d examples scanned from %d files with target %g", n_examples, n_files, target_);
}

void DiskBinDataSet::save(const char *filename)
{
	DiskXFile file(filename, "w");
	
	file.write(&n_examples, sizeof(int), 1);
	file.write(&n_inputs, sizeof(int), 1);

	for(int i=0; i<n_examples; i++)
	{
		setExample(i);
		file.write(inputs->frames[0], sizeof(real), n_inputs);
	}
}

void DiskBinDataSet::info(bool display_)
{
	message("DiskBinDataSet::info(): (only one example in memory)");
	print("\t+ n_examples = %d\n", n_examples);
	print("\t+ n_inputs = %d\n", n_inputs);
	print("\t+ n_targets = %d\n", n_targets);
		  	
	print("\n");

  	if(display_)
  	{
		for(int i=0; i<n_examples; i++)
	  	{
			setExample(i);

		  	// Targets
		  	print("[");
		  	for(int j=0; j<n_targets; j++)
				print(" %g ", targets->frames[0][j]);
		  	print("] ");

		  	// Inputs
		  	for(int j=0; j<n_inputs; j++)
			  	print("%g|", inputs->frames[0][j]);
		  	print("\n\n");
	  	}
  	}
}

DiskBinDataSet::~DiskBinDataSet()
{
}

}
