#include "ImageWeightedSumMachine.h"

namespace Torch {

ImageWeightedSumMachine::ImageWeightedSumMachine(Trainer **trainers_, int n_trainers_, MeasurerList** trainers_measurers_, real *weights_)
	: WeightedSumMachine(trainers_, n_trainers_, trainers_measurers_, weights_)
{
	outputs->resize(1);
	
	machines = (Machine **) allocator->alloc(n_trainers * sizeof(Machine *)); 
	for(int i = 0; i < n_trainers; i++)
		machines[i] = trainers[i]->machine;

}

ImageWeightedSumMachine::ImageWeightedSumMachine(Machine **machines_, int n_machines_)
{
   	// exact number of trainers/machines
  	n_trainers = n_machines_;
	n_trainers_trained = n_machines_;
	
	// copy the machines
	machines = machines_;

	// no trainers, no measurers
	trainers = NULL;
	trainers_measurers = NULL;

	// check if the output size of all the machines is the same
	n_outputs = machines[0]->outputs->frame_size;
	for(int i = 0; i < n_trainers; i++)
	{
		if(n_outputs != machines[i]->outputs->frame_size)
			error("ImageWeightedSumMachine: provided machines don't have the same output size!");
	}
	outputs = new(allocator) Sequence(1, n_outputs);

    	weights = (real *)allocator->alloc(n_trainers*sizeof(real));	
	for(int i = 0; i < n_trainers; i++) weights[i] = 0;
}

void ImageWeightedSumMachine::reset()
{
	for (int i=0;i<n_trainers;i++) machines[i]->reset();
	n_trainers_trained = 0;
}
	
void ImageWeightedSumMachine::forward(Sequence *inputs)
{
	// for an image machine the input sequence contains 1 frame: the image	
	real *dest_ = outputs->frames[0];
	for(int j = 0; j < n_outputs; j++) dest_[j] = 0;
    
	for(int i = 0; i < n_trainers_trained; i++)
	{
		machines[i]->forward(inputs);

		real z = weights[i];

		real *src_ = machines[i]->outputs->frames[0];
		for(int j = 0; j < n_outputs; j++) dest_[j] += z * src_[j];
	}
}

void ImageWeightedSumMachine::fastforward(Sequence *inputs)
{
    outputs->frames[0][0] = 0.0;
    for(int i = 0; i < n_trainers_trained; i++)
    {
	machines[i]->forward(inputs);
	outputs->frames[0][0] += weights[i] * machines[i]->outputs->frames[0][0];
    }
}

void ImageWeightedSumMachine::loadXFile(XFile *file)
{
	file->taggedRead(weights, sizeof(real), n_trainers, "WEIGHTS");  
	
	for (int i = 0; i < n_trainers; i++)
		machines[i]->loadXFile(file);
	n_trainers_trained = n_trainers;
}

void ImageWeightedSumMachine::saveXFile(XFile *file)
{
	file->taggedWrite(weights, sizeof(real), n_trainers_trained, "WEIGHTS");
	for (int i = 0; i < n_trainers_trained; i++)
		machines[i]->saveXFile(file);
}

ImageWeightedSumMachine::~ImageWeightedSumMachine()
{
}

}
