#ifndef VIDEO_FILE_INC
#define VIDEO_FILE_INC

#include "VideoSource.h"

namespace Torch {

/** This class is designed to handle a virtual pixmap from a file

    @author Sebastien Marcel (marcel@idiap.ch) 
    @version 2.0
    \Date
    @since 1.0
*/
class FrameVideoFile: public VirtualPixmap
{
private:
	// size of the pixmap
 	unsigned int width, height; 

	// the pixmap
	unsigned char *pixmap;

public:
	///
  	FrameVideoFile(unsigned int width_, unsigned int height_) { width = width_; height = height_; };

	///
  	virtual ~FrameVideoFile() {};

	///
  	unsigned int getwidth() const { return width; };

	///
  	unsigned int getheight() const { return height; };
  
	///
	const unsigned char *getpixmap() const { return pixmap; };

	///
	const int bytes_per_line() const { return width * 3; };

	///
	void copyBuffer(unsigned char *pixmap_) { pixmap = pixmap_; } ;
};

/** This class is designed to handle a file as a video source
	
    @author Sebastien Marcel (marcel@idiap.ch) 
    @version 2.0
    \Date
    @since 1.0
*/
class VideoFile : public VideoSource
{
protected:
	/// the name of the codec used
	char *codec_name;
	
	/// the virtual pixmap
	FrameVideoFile *frame;

	/// number of frames in the video source
	int nframes;

	/// frame rate of the video source
	real frate;

	/// current frame number
	int cframe;

	/// name of the video file
	char *filename;

	/// opening flag of the file
	char *open_flags;

public:
	/// the current image of the video file
	unsigned char *pixmap;

	//-----
	
	/** @name constructors */
	//@{
	/// empty constructor
  	VideoFile();

	/// construct a video source from a file
  	VideoFile(char *filename_, const char *open_flags_ = "r");
	//@}
  
	/// desctructor
  	virtual ~VideoFile();

	/// initizialize the video file
  	virtual bool initVideoSource();

	/// open the video file
  	virtual bool openVideoSource();

	/// read the current frame of the video file
  	virtual VirtualPixmap* getframe();

	/// close the video file
  	virtual bool closeVideoSource();

	//-----
	
	/// 
	unsigned int getwidth() { return width; };

	///
	unsigned int getheight() { return height; };

	//-----
	
	/// open the video file #filename_#
	virtual bool open(char *filename_, const char *open_flags_ = "r") = 0;

	/// read the current image from the file and fill #pixmap#
	virtual bool read() = 0;

	/// write #pixmap_# into a file
	virtual bool write(unsigned char *pixmap_) = 0;

	/// restart the video file at the beginning
	virtual bool rewindmovie() = 0;

	/** seek in the video file 
	
	    This function seeks at position #h#:#m#:#s# (assuming 25fps) and then read #f# frames.
	*/
	virtual int seekmovie(int h, int m, int s, int f) = 0;

	/// close the video file
	virtual bool close() = 0;

	//-----
	
	/// returns the frame rate in frame per second.
	real getframerate() { return frate; };

	/** returns the number of images (if possible).
	    
	    If #force#==true then it reads all the file to get the number of frames.
	*/
	int getnframes(bool force = false) { return nframes; };

	/// returns the name of the codec
	char *getcodec() { return codec_name; };
};

}

#endif
